#include "libwtref.h"

static void check_failed(void* ctx, unsigned int line)
{
	fprintf((FILE*)ctx, "Check failed @ line %u\n", line);
}

static void unres_exc(void* ctx, const char* name)
{
	fprintf((FILE*)ctx, "Unresolved exception: '%s'\n", name);
}

static void unres_func(void* ctx, const char* name, wt_u32 argc, wt_u32 retc,
	const wt_u32* argv, const wt_u32* retv)
{
	wt_u32 i;

	fprintf((FILE*)ctx, "Unresolved function: '%s', argc=%u, retc=%u\n",
		name, (unsigned int)argc, (unsigned int)retc);

	for(i=0;i<argc;++i)
	{
		fprintf((FILE*)ctx, "  argv[%u] = 0x%X\n", (unsigned int)i,
			(unsigned int)argv[i]);
	}

	for(i=0;i<retc;++i)
	{
		fprintf((FILE*)ctx, "  retv[%u] = 0x%X\n", (unsigned int)i,
			(unsigned int)retv[i]);
	}
}

static void unres_type(void* ctx, const char* name)
{
	fprintf((FILE*)ctx, "Unresolved type: '%s'\n", name);
}

static void unres_op(void* ctx, const char* name)
{
	fprintf((FILE*)ctx, "Unresolved operator: '%s'\n", name);
}

static const struct wtr_loaderops libwtref_loader_ops =
{
	.check_failed = check_failed,
	.unres_exc    = unres_exc,
	.unres_func   = unres_func,
	.unres_type   = unres_type,
	.unres_op     = unres_op,
};

wt_error libwtref_set_loader_ops(wt_profile vm, FILE* const out)
{
	return wtrProfileSetLoaderOps(vm, out, &libwtref_loader_ops);
}
