/* $Id: ast_foreachstatement.cpp 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "ast_foreachstatement.h"
#include "ast_variabledeclaration.h"
#include "ast_expression.h"
#include "ast_compound.h"
#include "../astvisitor.h"

wtast::ForeachStatement::~ForeachStatement()
{
	delete mVariable;
	delete mFrom;
	delete mTo;
	delete mContainer;
	delete mWhile;
	delete mWhich;
	delete mBody;
}

void wtast::ForeachStatement::Accept(AstVisitor* vis)
{
	if(vis->Visit(this))
	{
		mVariable->Accept(vis);

		if(mRangeBased)
		{
			mFrom->Accept(vis);
			mTo->Accept(vis);
		}
		else
		{
			mContainer->Accept(vis);
		}

		if(mWhile) { mWhile->Accept(vis); }
		if(mWhich) { mWhich->Accept(vis); }
		mBody->Accept(vis);

		vis->EndVisit(this);
	}
}

wtast::ForeachStatement* wtast::ForeachStatement::Copy()
{
	Expression* whil = 0;
	Expression* whic = 0;

	if(mWhile) { whil = mWhile->Copy(); }
	if(mWhich) { whic = mWhich->Copy(); }

	if(mRangeBased)
	{
		return new ForeachStatement(mVariable->Copy(), mFrom->Copy(), mTo->Copy(),
			whil, whic, mBody->Copy(), mInclusive, mReverse);
	}
	else
	{
		return new ForeachStatement(mVariable->Copy(), mContainer->Copy(),
			whil, whic, mBody->Copy(), mReverse);
	}
}
