/* $Id: ast_recordfield.h 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_RECORDFIELD_H
#define WTPARSER_AST_RECORDFIELD_H

#include <string>
#include "../astnode.h"
#include "../../wtruntime/wtruntime.h"

namespace wtast
{

class Type;

class RecordField : public AstNode
{
	Type*       mType;
	std::string mName;
	wt_u32      mOffset;

public:
	RecordField(Type* type, const std::string & name)
		: AstNode()
		, mType(type)
		, mName(name)
		, mOffset(0)
	{
		// ok
	}

	~RecordField();

	void Accept(AstVisitor* vis);

	RecordField* Copy();

	Kind GetKind() const
	{
		return RECORDFIELD;
	}

	Type* GetType()
	{
		return mType;
	}

	std::string & GetName()
	{
		return mName;
	}

	wt_u32 GetOffset() const
	{
		return mOffset;
	}

	friend class RecordType;

private:

	void SetOffset(wt_u32 offs)
	{
		mOffset = offs;
	}
};

} // namespace wtast

#endif // WTPARSER_AST_RECORDFIELD_H
