/* $Id: ast_type.h 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_TYPE_H
#define WTPARSER_AST_TYPE_H

#include "../astnode.h"
#include "ast_typeinfo.h"
#include "../../wtruntime/wtruntime.h"
#include <string>

namespace wtast
{

class Type : public TypeInfo
{
public:
	Type()
		: TypeInfo()
	{
		// ok
	}

	~Type()
	{
		// ok
	}

	virtual Type* Copy() = 0;

	virtual std::string GetName() const = 0; // from wtast::TypeInfo
	virtual wt_u32 GetTypeId() const = 0;
	virtual wt_u32 GetLoadSize() const = 0;
	virtual bool HasObject() const { return IsObject(); }
	virtual bool IsObject() const = 0;
	virtual bool IsSequence() const { return false; }
	virtual void SetMutable() { }
	virtual bool IsMutable() const { return 0!=(WTTF_MUTABLE & GetTypeId()); }
	virtual bool IsDefaultConstructible() const { return true; }
	virtual bool IsCopyable() const { return true; }
	virtual bool IsFunctionType() const { return false; }
	virtual bool HasFunctionType() const { return false; }
};

} // namespace wtast

#endif // WTPARSER_AST_TYPE_H
