#include "config_private.h"
#include "dynstack.h"
#include "profile.h"
#include <string.h>

wt_bool wtr_dynstack_init(struct wtr_dynstack* const stk,
	struct wtr_profile* const prof, wt_u32 minsize, wt_u32 maxsize,
	wt_u32 limit)
{
	stk->sp       = 0;
	stk->size     = minsize;
	stk->min_size = minsize;
	stk->max_size = maxsize;
	stk->limit    = limit;
	stk->profile  = prof;

	if(NULL==(stk->data=(struct wtr_objentry*)wtr_profile_sys_malloc(prof,
		sizeof(struct wtr_objentry)*minsize)))
	{
		return WT_FALSE;
	}

	return WT_TRUE;
}

void wtr_dynstack_free(struct wtr_dynstack* const stk)
{
	if(NULL!=stk->data)
	{
		wtr_profile_sys_free(stk->profile, stk->data);
		stk->data = NULL;
	}
}

wt_bool wtr_dynstack_push(struct wtr_dynstack* const stk,
	struct wtr_objentry* const entry)
{
	if(stk->sp<stk->size)
	{
		/* branch prediction optimization */
	}
	else
	{
		const size_t raw_size = sizeof(struct wtr_objentry)*stk->size;

		if(stk->size==stk->max_size)
		{
			return WT_FALSE;
		}

		struct wtr_objentry* const new_arr =
			(struct wtr_objentry*)wtr_profile_sys_realloc(stk->profile,
				stk->data, raw_size, raw_size<<1);

		if(NULL==new_arr)
		{
			return WT_FALSE;
		}

		stk->data    = new_arr;
		stk->size  <<= 1;
		stk->limit <<= 1;
	}

	stk->data[stk->sp++] = *entry;

	return WT_TRUE;
}

struct wtr_objentry* wtr_dynstack_top(struct wtr_dynstack* const stk)
{
	if(0<stk->sp)
	{
		return stk->data + stk->sp - 1;
	}

	return NULL;
}

wt_bool wtr_dynstack_pop(struct wtr_dynstack* const stk)
{
	if(stk->sp)
	{
		--stk->sp;

		if(stk->sp>=stk->limit || stk->size<=stk->min_size)
		{
			return WT_TRUE;
		}
		else
		{
			const size_t raw_size = sizeof(struct wtr_objentry)*stk->size;

			struct wtr_objentry* const new_arr =
				(struct wtr_objentry*)wtr_profile_sys_realloc(stk->profile,
					stk->data, raw_size, raw_size/2);

			if(NULL!=new_arr)
			{
				stk->data    = new_arr;
				stk->size  >>= 1;
				stk->limit >>= 1;
			}

			return WT_TRUE;
		}
	}

	return WT_FALSE;
}
