#include "config_private.h"
#include "operator.h"
#include "program.h"
#include "profile.h"
#include <string.h>

struct wtr_operator* wtr_operator_new(struct wtr_program* program, const char* name, wt_u32 prec, wt_u32 assoc)
{
	const wt_u32               nlen = strlen(name) + 1;
	struct wtr_operator* const op   = (struct wtr_operator*)wtr_profile_sys_malloc(program->profile, sizeof(struct wtr_operator)+nlen);

	if(NULL!=op)
	{
		char* after = (char*)(op + 1);

		op->precedence    = prec;
		op->associativity = assoc;
		op->refcount      = 0;
		op->name          = after;
		op->program       = program;

		strcpy(after, name);
	}

	return op;
}

void wtr_operator_release(struct wtr_operator* op)
{
	struct wtr_profile* const profile = op->program->profile;

	memset(op, 0, sizeof(struct wtr_operator));
	wtr_profile_sys_free(profile, op);
}

int wtr_operator_cmp(const struct wtr_operator* lhs, const struct wtr_operator* rhs)
{
	if(lhs->precedence<rhs->precedence)
	{
		return -1;
	}

	if(lhs->precedence>rhs->precedence)
	{
		return +1;
	}

	if(lhs->associativity<rhs->associativity)
	{
		return -1;
	}

	if(lhs->associativity>rhs->associativity)
	{
		return +1;
	}

	return strcmp(lhs->name, rhs->name);
}
