#ifndef WTRUNTIME_PROGRAM_H
#define WTRUNTIME_PROGRAM_H

#include "config.h"
#include "types.h"
#include "vector.h"
#include "libvmma/libvmma.h"
#include <stdlib.h>

#define WTR_PTR_TO_VMADDR(_prog_,_ptr_)   ((wt_vmptr)(((wt_u8*)(_ptr_)) - ((wt_u8*)(_prog_)->memory)))
#define WTR_PTR_FROM_VMADDR(_prog_,_vma_) ((void*)(((wt_u8*)(_prog_)->memory) + (_vma_)))

struct wtr_profile;
struct wtr_program;
struct wtr_exception;
struct wtr_function;
struct wtr_type;
struct wtr_typeslot;
struct wtr_operator;

struct wtr_program
{
	struct wtr_node        link_programs;         /* link in the program list of the profile */
	struct wtr_profile*    profile;               /* the program belongs to this profile */
	wt_uintptr             refcount;              /* number of references to this program (due to other programs, or function handles) -- starts at 1 (self-reference) */
	struct vmma_client*    alloc_client;          /* program-specific memory allocation context */
	wt_u8*                 memory;                /* profile.memory */
	const wt_u16*          code;                  /* profile.memory as read-only code */
	wt_u8*                 program_data;          /* data allocated for the program (that the binary specified to the loader) */
	wt_uintptr             program_size;          /* size of 'program_data' in octets */
	void*                  user_ptr;              /* an additional user pointer, this is opaque to the VM */
	wt_u32                 pc_base;               /* program-local PC base address relative to program.code */
	wt_u32                 pc_ctor;               /* PC of the ctor{} (program-relative address; WTR_NULLPC if not exists */
	wt_u32                 pc_dtor;               /* PC of the dtor{} (program-relative address; WTR_NULLPC if not exists */
	wt_u32                 pc_main;               /* entry point of the program (in practice PC of main{}), program-relative address */
	wt_vmptr               base_address;          /* VM address of 'program_data' */
	size_t                 num_ref_types;         /* number of referenced types */
	struct wtr_typeslot*   referenced_types;      /* array of referenced types, stored in the data area of the program */
	struct wtr_vector      referenced_functions;
	struct wtr_vector      referenced_exceptions;
	struct wtr_vector      own_exceptions;
	struct wtr_vector      referenced_operators;
	struct wtr_vector      own_operators;
	struct wtr_vector      own_types;
	struct wtr_vector      metadata_vec;
};

/* metadata structure, key string and value string are allocated in a */
/* single memory block, thus freeing a metadata is achive by calling */
/* wtr_profile_sys_free(vm, metadata_ptr) */
struct wtr_metadata
{
	const char* key;
	const char* val;
};


/* resolve a type in the context of the given program */
wt_bool wtr_program_resolvetype(struct wtr_program* prog, wt_u32* p_type);


/* same as strdup(), but allocates memory from the given program */
char* wtr_program_strdup(struct wtr_program* prog, const char* str);


/* memory allocation */
void* wtr_program_malloc(struct wtr_program* prog, size_t size);
void  wtr_program_free  (struct wtr_program* prog, void* memory);


void wtr_program_destroy(struct wtr_profile* const profile,
	struct wtr_program* const program);


/* free up referenced and owned objects and their containers -- can be used */
/* from the loader if 'ownfunc' is not NULL */
void wtr_program_free_refd_owned(struct wtr_profile* const profile,
	struct wtr_program* const program, struct wtr_vector* const ownfunc);


#endif /* WTRUNTIME_PROGRAM_H */
